######################## TCL FILE ##############################
# Description
# Command file generator for machine: RFP-6K
# 5 axis cutting Machine Tool
# CNC control system: ALPHA
# Software version: 2.0
#
# Revisions
#
#   Date            		Who        		Reason
#  07--2004		- 	    First Release
# $HISTORY$
#
################################################################################

 set cam_post_dir [MOM_ask_env_var UGII_CAM_POST_DIR]
 set cam_debug_dir [MOM_ask_env_var UGII_CAM_DEBUG_DIR]

#source "[MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_review.tcl" ;
  MOM_set_debug_mode OFF

#_______________________________________________________________________________
#The following command invokes the warning mode and the default settings.
#_______________________________________________________________________________

  source "[MOM_ask_env_var UGII_CAM_POST_DIR]ugpost_base.tcl"

#_______________________________________________________________________________
#The following file consists of default values set for G & M codes
#_______________________________________________________________________________

#---------------- M code declaration

  set mom_sys_program_stop_code                 0
  set mom_sys_optional_stop_code                1
  set mom_sys_end_of_program_code               2

  set mom_sys_spindle_direction_code(NONE)      3
  set mom_sys_spindle_direction_code(CLW)       3
  set mom_sys_spindle_direction_code(CCLW)      4
  set mom_sys_spindle_direction_code(OFF)       5

  set mom_sys_coolant_code(FLOOD)               8
  set mom_sys_coolant_code(ON)                  8
  set mom_sys_coolant_code(OFF)                 9
  set mom_sys_coolant_code(TAP)                 8
  set mom_sys_coolant_code(MIST)                7

  set mom_sys_clamp_code(ON)                    10
  set mom_sys_clamp_code(OFF)                   11
  set mom_sys_clamp_code(AXISON)                45
  set mom_sys_clamp_code(AXISOFF)               46

  set mom_sys_rotary_tool_direction_code(CLW)   13
  set mom_sys_rotary_tool_direction_code(CCLW)  14

  set mom_sys_tailstock_out_code                25
  set mom_sys_tailstock_in_code                 26

  set mom_sys_program_end_code                  30
  set mom_sys_c_axis_on_code                    45
  set mom_sys_c_axis_off_code                   46
  set mom_sys_spindle_brake_clamp_code          68
  set mom_sys_spindle_brake_unclamp_code        69
  set mom_sys_tailstock_clamp_code              78
  set mom_sys_tailstock_unclamp_code            79

#---------------- G code declaration

  set mom_sys_rapid_code                     0
  set mom_sys_linear_code                    1
  set mom_sys_circle_code(CLW)               2
  set mom_sys_circle_code(CCLW)              3
  set mom_sys_delay_code(REVOLUTIONS)        4
  set mom_sys_delay_code(SECONDS)            4
  set mom_sys_polar_mode_on                  12.1
  set mom_sys_polar_mode_off                 13.1
  set mom_sys_cutcom_plane_code(XY)          17
  set mom_sys_cutcom_plane_code(ZX)          18
  set mom_sys_cutcom_plane_code(YZ)          19
  set mom_sys_inch_code                      20
  set mom_sys_metric_code                    21
  set mom_sys_ref_point_return_code          28
  set mom_sys_cutcom_code(OFF)               40
  set mom_sys_cutcom_code(LEFT)              41
  set mom_sys_cutcom_code(RIGHT)             42
  set mom_sys_spindle_max_rpm_code           50
  set mom_sys_wcs_change_code                0
  set mom_work_coordinate_number             0
#
  set mom_sys_mirror_image_on                68
  set mom_sys_mirror_image_off               69
#
  set mom_sys_cycle_off                      80
  set mom_sys_cycle_drill_code               81
#  G82 is Spot Facing Cycle,C-Bore and C-Sink
  set mom_sys_cycle_drill_dwell_code         82
  set mom_sys_cycle_drill_deep_code          83
  set mom_sys_cycle_breakchip_code           83.1

  set mom_sys_cycle_tap_code                 84
  set mom_sys_cycle_tap_code_left            84.1

  set mom_sys_cycle_bore_code                85
#  G86 is Reaming Cycle
  set mom_sys_cycle_bore_drag_code           86
  set mom_sys_cycle_bore_dwell_code          89

  set mom_sys_spindle_mode_code(SFM)         96
  set mom_sys_spindle_mode_code(SMM)         96
  set mom_sys_spindle_mode_code(RPM)         97

  set mom_sys_feedrate_code(IPM)             98
  set mom_sys_feedrate_code(MMPM)            98
  set mom_sys_feedrate_code(IPR)             99
  set mom_sys_feedrate_code(MMPR)            99

#----------------------------- VARIABLES

  set rad2deg [expr 90.0 / asin(1.0)]
  set PI [expr 2.0 * asin(1.0)]                    ; #value PI
  set initial_move 0
  set linear_move 0
  set spindle_speed 0
  set spindle_speed_1 0
  set korr_move  0
  set nn 0

#_______________________________________________________________________________
#
# Kinematic Variable Declaration
#_______________________________________________________________________________

# Note:-  T means ON & F means OFF

  set mom_kin_machine_type                 5_axis_dual_head
  set mom_sys_opskip_block_leader          "/"
  set mom_kin_linearization_flag           T
  set mom_kin_linearization_tol            0.01
  set mom_kin_coordinate_type              CARTESIAN
  set mom_kin_machine_resolution           0.01
  set mom_kin_max_arc_radius               99999.999
  set mom_kin_min_arc_length               0.025
  set mom_kin_min_arc_radius               0.016 ; # 0.0002
  set mom_kin_arc_valid_plane              XY
  set mom_kin_arc_output_mode              QUADRANT ; # QUADRANT ; LINEAR ; FULL_CIRCLE
  set mom_kin_rapid_feed_rate              5000.0
  set mom_kin_max_dmp                      7200.0

  #-----------------------------------------------------------------------------
  set mom_sys_leader(fourth_axis)          A
  set mom_sys_leader(fifth_axis)           
  set mom_kin_4th_axis_plane               YZ
  set mom_kin_4th_axis_direction           MAGNITUDE_DETERIMINES_DIRECTION ; # SIGN_DETERMINES_DIRECTION
  set mom_kin_4th_axis_rotation            reverse
  set mom_kin_4th_axis_zero                0.0
  set mom_kin_4th_axis_center_offset(0)    0.0
  set mom_kin_4th_axis_center_offset(1)    0.0
  set mom_kin_4th_axis_center_offset(2)    0.0
  set mom_kin_4th_axis_min_limit         -30.0 ; # -20.0 -21.5 -30.0 -90.0
  set mom_kin_4th_axis_max_limit          30.0 ; # 20.0 21.5 30.0 90.0

  set mom_kin_5th_axis_plane               XY

  set mom_kin_5th_axis_direction           MAGNITUDE_DETERIMINES_DIRECTION ; # SIGN_DETERMINES_DIRECTION
  set mom_kin_5th_axis_rotation            standard
  set mom_kin_5th_axis_center_offset(0)    0.0
  set mom_kin_5th_axis_center_offset(1)    0.0
  set mom_kin_5th_axis_center_offset(2)    0.0
  set mom_kin_5th_axis_min_limit          -270.0 ; # -20.0 -21.5 -30.0 -90.0
  set mom_kin_5th_axis_max_limit           270.0 ; # 20.0 21.5 30.0 90.0

  set mom_kin_pivot_gauge_offset           0

     set mom_kin_4th_axis_min       0
     set mom_kin_5th_axis_min       0

     set mom_kin_4th_axis_max       0
     set mom_kin_5th_axis_max       0
#_______________________________________________________________________________
# Global Variable Declaration for Listing File Output
#_______________________________________________________________________________
  set mom_sys_list_file_columns             132
  set mom_sys_list_file_rows                 80
  set mom_sys_commentary_output              ON
  set mom_sys_group_output                   OFF
  set mom_sys_list_output                    ON
  set mom_sys_warning_output                 ON
  set mom_event_time                         0

  set output_mode                           "CARTESIAN"
  set yaxis_status                           1
  set count  0
  set mom_tool_number                        1
  set mom_spindle_rpm                        0
  set mom_tool_gage_length                   0
  set operation_flag                         0
  set operation_count                        0
  set rapid_flag                             0
  set g_adjust                               0
  set dinst     0

#===============================================================================
  #    
  set first_tool 0

#=============================================================
proc MOM_origin {} {
#=============================================================
  global mom_origin  mom_origin_text
  global mom_warning_info
}

proc MOM_msys {} {
#_______________________________________________________________________________
#
# This procedure is called for a MSYS statement
#_______________________________________________________________________________


  global mom_warning_info
  global mom_origin
  global mom_tool_offset_defined  mom_tool_offset
  global first_tool

    set first_tool 0


}

proc MOM_from_move {} {
#_______________________________________________________________________________
#
# This procedure is called for a FROM statement
#_______________________________________________________________________________
  MOM_do_template from CREATE
}


proc MOM_gohome_move {} {
#_______________________________________________________________________________
#
# This procedure is called for a GOHOME move
#_______________________________________________________________________________
  MOM_do_template gohome
}


proc MOM_start_of_program {} {
#
# This procedure is executed at the very begining of the program.
# It gets called before any command is read from the task file.
#_______________________________________________________________________________
  global mom_coolant_mode  mom_coolant_status
  global mom_cutcom_mode   mom_cutcom_status

#---- Unsetting defult setting for variables
  catch {unset mom_coolant_mode}
  catch {unset mom_coolant_status}
  catch {unset mom_cutcom_mode}
  catch {unset mom_cutcom_status}

 OPEN_files ; #------- The following procedure opens the warning and listing files
 LIST_FILE_HEADER ; #------- The following procedure lists the header information in commentary data

#------- These are to be obtained for the user before Postprocessing
#  set str "\n    '' - 5- . ('   - A,C') \n\n"
#  MOM_output_to_listing_device $str  ;

  global  mom_warning_info
  global  mom_output_file_basename
  global  mom_logname  mom_date  mom_machine_name

  set strname "" ;
  if [llength [info commands PB_part_name] ] {  set strname [ PB_part_name ] ;  }

  set strmaterial "" ;
  if [llength [info commands PB_part_material] ] { set strmaterial [ PB_part_material ] ; }

  #MOM_suppress once N
  MOM_set_seq_off
  MOM_output_literal "; Part: $strname" ;
  MOM_output_literal "; Material: $strmaterial" ;
  MOM_output_literal "; $mom_output_file_basename"
  MOM_output_literal "; $mom_logname, Date:$mom_date"
  MOM_output_literal "; MACHINE: RFP-6k"
  MOM_output_literal "    " ;
  MOM_output_literal "%"
  MOM_set_seq_on
  MOM_output_literal "(UAO,1)"

}


proc MOM_end_of_program {} {
#_______________________________________________________________________________
# This procedure is executed at the end of the program after all
# the paths are traversed.
#_______________________________________________________________________________
  global mom_tool_number

  global list_file
  global  mom_kin_4th_axis_min mom_kin_5th_axis_min
  global  mom_kin_4th_axis_max mom_kin_5th_axis_max

#  MOM_output_literal "T$mom_tool_number.0 M6"
           MOM_output_literal "(UAO,0)"
           MOM_output_literal "G0G90X0Y0Z0A0C0"
           MOM_do_template end_of_program

#---- The following procedure lists the tool list with time in commentary data

  if [llength [info commands PB_CMD_limit_angle_AB_saveout] ] {
    PB_CMD_limit_angle_AB_saveout
  }

  LIST_FILE_TRAILER ; #---- The following procedure closes the warning and listing files
  CLOSE_files
}


#=============================================================
proc MOM_machine_mode {} {
#=============================================================

}


proc MOM_start_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of each path in the task
# It gets called after all the general parameters of the path are loaded
# like the path name and the tool name.
#_______________________________________________________________________________

  global mom_tool_name
  global mom_path_name
#  MOM_force once F
#  MOM_output_literal "(----------------------------------------------------)"
#  MOM_output_literal "(Start path: $mom_path_name with tool: $mom_tool_name)"
#  MOM_output_literal "(----------------------------------------------------)"
  MOM_force once G_motion F

  ;#      ( )
  global  mom_update_post_cmds_from_tool

  global mom_imported_path_status
  global mom_kin_arc_output_mode mom_operation_name
  if {[info exists mom_imported_path_status]==1} {
     if {"YES"==[string toupper "$mom_imported_path_status"]} {
       set mom_warning_info "WARNING: =$mom_operation_name -   ( -   !)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set mom_kin_arc_output_mode  "QUADRANT"  ;   #QUADRANT  FULL_CIRCLE , LINEAR
      } else {
        set mom_kin_arc_output_mode  "FULL_CIRCLE"  ;   #QUADRANT  FULL_CIRCLE , LINEAR
       }
     MOM_reload_kinematics
   }

}


proc MOM_end_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the end of a path.
#_______________________________________________________________________________

}


proc MOM_tool_change {} {
#_______________________________________________________________________________
# This procedure is executed at any time there is a tool change.
# It gets called after the tool change data is loaded like the tool number
# and the register numbers.
#_______________________________________________________________________________

  global mom_path_name  mom_operation_type
  global mom_tool_name  mom_tool_number  mom_tool_adjust_register

#---- The put end of path blocks for the previous path
#-------- These are New Tool Startup Comments
#  MOM_suppress always N
#  MOM_output_literal "(PATH NAME=$mom_path_name)"
#  MOM_output_literal "(PATH DESC=$mom_operation_type)"
#  MOM_output_literal "(TOOL NUMB=$mom_tool_number)"
#  MOM_output_literal "(TOOL DESC=$mom_tool_name)"
#  MOM_output_literal "(TOOL OFST=$mom_tool_adjust_register)"
#  MOM_suppress off N
# MOM_do_template  tool_change
}


proc MOM_first_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the first operation, with a tool, in the task.
#_______________________________________________________________________________
  global mom_path_name  mom_operation_type
  global mom_tool_name  mom_tool_number  mom_tool_adjust_register

  global first_tool
  set first_tool 1

}


proc MOM_initial_move {} {
#_______________________________________________________________________________
# This procedure is executed for the initial move of each operation.  It
# assumes the tool is moving from a safe position at rapid to the start of
# the operation.
#_______________________________________________________________________________
  global mom_coolant_mode  mom_coolant_status

  global output_mode  initial_move
  global feed
  set initial_move 1

#------- These are New Tool Start blocks:- Move to Tool Change Positon
#  MOM_force once G_motion G_cutcom G_feed Z
#  MOM_force once   X Y Z
#  MOM_do_template Start_up
#  MOM_output_literal "M19"
#  MOM_output_literal "M6"

#  MOM_do_template home
#  MOM_do_template tool_change

#  MOM_force once  G_motion X Y Z M_cool
  if {[info exists mom_coolant_mode]} { set mom_coolant_status $mom_coolant_mode ; }

#  MOM_output_literal "G28H0"
#  MOM_output_literal "G0C0"
#  global mom_sys_wcs_change_code mom_work_coordinate_number
#  set aa  [expr $mom_sys_wcs_change_code+$mom_work_coordinate_number]
#  MOM_output_literal "(UAO,$aa)"
#  MOM_output_literal "(UAO,1)"
#  
   MOM_force once  G_motion X Y Z
   MOM_do_template rapid_xyz
#  MOM_do_template rapid_z
#  global mom_spindle_mode  mom_spindle_speed
#  if {$mom_spindle_speed != 0} { MOM_do_template rotary_start }

}


##########################################################################################
#=============================================================
proc MOM_first_move {} {
#=============================================================
  global mom_motion_event
  global mom_coolant_mode  mom_coolant_status
  if {[info exists mom_coolant_mode]} {  set mom_coolant_status $mom_coolant_mode ; }
  catch {MOM_$mom_motion_event}
}


#=============================================================
proc MOM_before_motion { } {
#=============================================================
  global mom_motion_event mom_motion_type

    FEEDRATE_SET

    switch $mom_motion_type {
      ENGAGE   { ; }
      APPROACH { ; }
      FIRSTCUT { ; }
      RETRACT  { ; }
      RETURN   { ; }
    }

    if [llength [info commands PB_CMD_kin_before_motion] ] { PB_CMD_kin_before_motion }
    if [llength [info commands PB_CMD_before_motion] ]     { PB_CMD_before_motion }
}


#=============================================================
proc PB_CMD_before_motion { } {
#=============================================================
  global mom_motion_type
  global mom_current_motion
  global mom_pos
  global mom_warning_info mom_seqnum warning_1 nn

  global g_plane ;   set g_plane [CHECK_PLANE]

#           ( +-270 .)
#          ( +-30 .)

  if {[info exists mom_warning_info]== "unable to determine valid rotary positions" } {
   incr nn
   set warning_1($nn) [format "; :   %.0f      (+-30 )" $mom_seqnum ]
   set mom_warning_info ""
	}
  if {[info exists mom_warning_info]== "ROTARY CROSSING LIMIT."   } {
   incr nn
   set warning_1($nn) [format "; :   %.0f      C" $mom_seqnum ]
   set mom_warning_info ""
  }
}


###############################################################################

########################
# Redefine FEEDRATE_SET -     MMPM ;   - 
########################
if [llength [info commands ugpost_FEEDRATE_SET] ] {
   rename ugpost_FEEDRATE_SET ""
}

if [llength [info commands FEEDRATE_SET] ] {
   rename FEEDRATE_SET ugpost_FEEDRATE_SET
} else {
   proc ugpost_FEEDRATE_SET {} {}
}


#=============================================================
proc FEEDRATE_SET {} {
#=============================================================
   if [llength [info commands PB_CMD_kin_feedrate_set] ] {
      PB_CMD_kin_feedrate_set
   } else {
      ugpost_FEEDRATE_SET
   }
}

proc ASK_FEEDRATE_FPM {} {
#_______________________________________________________________________________
# returns feed rate IN or MM per min
#_______________________________________________________________________________
  global mom_feed_rate

  if {[hiset mom_feed_rate]} {
    global mom_kin_rapid_feed_rate
    if {$mom_feed_rate>=$mom_kin_rapid_feed_rate} {
  	set mom_feed_rate $mom_kin_rapid_feed_rate
    }
    return $mom_feed_rate
  } else {
    set mom_warning_info "FEED RATE UNDEFINED" ; MOM_catch_warning ; return 0.0
  }
}


set feed 0.0 ; set feed_mode MMPM

#=============================================================
proc PB_CMD_kin_feedrate_set { } {
#=============================================================
# This procedure supercedes the functionalites provided by the
# FEEDRATE_SET in ugpost_base.tcl.  Post Builder automatically
# generates proper call sequences to this procedure in the
# Event handlers.
#
# This procedure must be used in conjunction with ugpost_base.tcl.
#
  global   feed com_feed_rate
  global   mom_feed_rate_output_mode super_feed_mode feed_mode
  global   mom_cycle_feed_rate_mode mom_cycle_feed_rate
  global   mom_cycle_feed_rate_per_rev
  global   mom_motion_type
  global   mom_warning_info
  global   Feed_IPM Feed_IPR Feed_MMPM Feed_MMPR Feed_INV
  global   mom_sys_feed_param
  global   mom_sys_cycle_feed_mode

  set super_feed_mode $mom_feed_rate_output_mode

  set f_pm [ASK_FEEDRATE_FPM] ; set f_pr [ASK_FEEDRATE_FPR]

  switch $mom_motion_type {

    CYCLE {
      if [info exists mom_sys_cycle_feed_mode] {
         if { $mom_sys_cycle_feed_mode != "Auto" } {
            set mom_cycle_feed_rate_mode $mom_sys_cycle_feed_mode
         }
      }
      if {[hiset mom_cycle_feed_rate_mode]} { set super_feed_mode $mom_cycle_feed_rate_mode }
      if {[hiset mom_cycle_feed_rate]} { set f_pm $mom_cycle_feed_rate }
      if {[hiset mom_cycle_feed_rate_per_rev]} { set f_pr $mom_cycle_feed_rate_per_rev }
    }

    FROM -
    RETRACT -
    RETURN -
    LIFT -
    TRAVERSAL -
    GOHOME  -
    GOHOME_DEFAULT -
    RAPID {
      SUPER_FEED_MODE_SET RAPID
    }

    default {
      if {[EQ_is_zero $f_pm] && [EQ_is_zero $f_pr]} {
        SUPER_FEED_MODE_SET RAPID
      } else {
        SUPER_FEED_MODE_SET CONTOUR
      }
    }
  }

  set feed_mode $super_feed_mode

 # Adjust feedrate format per Post output unit again.
  global mom_kin_output_unit
  if { $mom_kin_output_unit == "IN" } {
     switch $feed_mode {
       MMPM {
         set feed_mode "IPM"
         #CATCH_WARNING "Feedrate mode MMPM changed to IPM"
       }
       MMPR {
         set feed_mode "IPR"
         #CATCH_WARNING "Feedrate mode MMPR changed to IPR"
       }
     }
    #CATCH_WARNING "      - "
    #MOM_abort "\n\n\t      - \n\n"
  } else {
     switch $feed_mode {
       IPM {
         set feed_mode "MMPM"
         #CATCH_WARNING "Feedrate mode IPM changed to MMPM"
       }
       IPR {
         set feed_mode "MMPR"
         #CATCH_WARNING "Feedrate mode IPR changed to MMPR"
       }
     }
  }

 global  mom_path_name

  switch $feed_mode {
    IPM     -
    MMPM    { set feed $f_pm }
    IPR     -
    MMPR    {
    	        CATCH_WARNING "    c   - MMPR. Operation $mom_path_name \nFeedrate mode MMPR changed to MMPM"
    	        set feed $f_pm
    	        set feed_mode "MMPM"
     }
    DPM     {
    	#returns feed rate in degrees per min
    	CATCH_WARNING "    c   - DPM. Operation $mom_path_name "
    	set feed [PB_CMD_FEEDRATE_DPM]
     }
    FRN     -
    INVERSE {
    	# returns feed rate number in inverse time
    	CATCH_WARNING "    c   - FRN , INVERSE . Operation $mom_path_name "
    	set feed [PB_CMD_FEEDRATE_NUMBER]
     }
    default { set mom_warning_info "INVALID FEED RATE MODE" ; MOM_catch_warning ; return }
  }


 # Post Builder provided format for the current mode:
  if [info exists mom_sys_feed_param(${feed_mode},format)] {
    MOM_set_address_format F $mom_sys_feed_param(${feed_mode},format)
  } else {
    switch $feed_mode {
      IPM     -
      MMPM    -
      IPR     -
      MMPR    -
      DPM     -
      FRN     { MOM_set_address_format F Feed_${feed_mode} }
      INVERSE { MOM_set_address_format F Feed_INV }
    }
  }

 # Commentary output
  set com_feed_rate $f_pm


 # Execute user's commnad, if any.
  if [llength [info commands "PB_CMD_FEEDRATE_SET"]] {
     PB_CMD_FEEDRATE_SET
  }
}


proc MOM_linear_move {} {
#_______________________________________________________________________________
# This procedure is executed for each linear move. It gets called after
# the positioning varables, position and tool axis (X,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________
  global output_mode  linear_move

  global mom_cutcom_mode  mom_cutcom_status
  if {[info exists mom_cutcom_mode]} {  set mom_cutcom_status $mom_cutcom_mode }

  global mom_warning_info
  global mom_out_angle_pos
  global mom_kin_4th_axis_min mom_kin_5th_axis_min
  global mom_kin_4th_axis_max mom_kin_5th_axis_max
  #   4-  5-    (+-20  )
  set fl 0
  if {$mom_out_angle_pos(0)>30.0} {
    set mom_warning_info "      20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }
  if {$mom_out_angle_pos(0)<-30.0} {
    set mom_warning_info "      -20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }
  if {$mom_out_angle_pos(1)>270.0} {
    set mom_warning_info " C     20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }
  if {$mom_out_angle_pos(1)<-270.0} {
    set mom_warning_info " C     -20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }

  if {$mom_out_angle_pos(0)>0.0} {
    if {$mom_out_angle_pos(0)>$mom_kin_4th_axis_max} { set mom_kin_4th_axis_max $mom_out_angle_pos(0) ; }
  } elseif {$mom_out_angle_pos(0)<0.0} {
           if {$mom_out_angle_pos(0)<$mom_kin_4th_axis_min} { set mom_kin_4th_axis_min $mom_out_angle_pos(0) ; }
        }

  if {$mom_out_angle_pos(1)>0.0} {
    if {$mom_out_angle_pos(1)>$mom_kin_5th_axis_max} { set mom_kin_5th_axis_max $mom_out_angle_pos(1) ; }
  } elseif {$mom_out_angle_pos(1)<0.0} {
  	   if {$mom_out_angle_pos(1)<$mom_kin_5th_axis_min} { set mom_kin_5th_axis_min $mom_out_angle_pos(1) ; }
         }

  MOM_do_template linear

  if {$fl==1} {
     #MOM_output_to_listing_device $mom_warning_info
     MOM_catch_warning
     MOM_set_line_leader off " !Error! "
     set fl 0
  }


}


proc MOM_circular_move {} {
#_______________________________________________________________________________
# This procedure is executed for each circular move. It gets called after
# the circle data are loaded (end point and center).
#_______________________________________________________________________________
  global mom_pos_arc_axis  mom_arc_radius
  global mom_pos  mom_prev_pos
  global mom_path_name
  global mom_cutcom_mode   mom_cutcom_status
  global mom_motion_distance

  global g_plane operation PI

  switch $g_plane {
     "17" {
          MOM_suppress once K ; MOM_force always  I J
          set tmp0 [expr abs($mom_pos(0)-$mom_prev_pos(0)) ]
          set tmp1 [expr abs($mom_pos(1)-$mom_prev_pos(1)) ]
          set f [ expr (2.0*$PI*$mom_arc_radius) ]
          set tmp [expr abs($f-$mom_motion_distance) ]
          #  MOM_output_literal "fff=$mom_motion_distance f=$f "
          if {$tmp0<0.015 && $tmp1<0.015} {
             if {[EQ_is_gt $tmp 0.0000001]} {
               if {[EQ_is_gt $tmp 0.1]} { MOM_linear_move ; return ;  }
              }
           }
        }
     "18" {
          MOM_suppress once J
          MOM_output_to_listing_device "WARNING: Path :- $mom_path_name"
          MOM_output_to_listing_device "WARNING: Cannot do circles in G18 plane"
        }
    "19" {
          MOM_output_to_listing_device "WARNING: Path :- $mom_path_name"
          MOM_output_to_listing_device "WARNING: Cannot do circles in G19 plane"
        }
  }

  if {[info exists mom_cutcom_mode]} { set mom_cutcom_status $mom_cutcom_mode }

# MOM_do_template plane
  MOM_do_template circle
}


proc MOM_rapid_move {} {
#_______________________________________________________________________________
# This procedure is executed for each rapid move. It gets called after
# the positioning varables, position and tool axis (X,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________
  global mom_coolant_mode  mom_coolant_status
  global mom_motion_type

  if {[info exists mom_coolant_mode]} { set mom_coolant_status $mom_coolant_mode }

  global mom_warning_info
  global mom_out_angle_pos
  global mom_kin_4th_axis_min mom_kin_5th_axis_min
  global mom_kin_4th_axis_max mom_kin_5th_axis_max
  #   4-  5-    (+-20  )
  set fl 0
  if {$mom_out_angle_pos(0)>30.0} {
    set mom_warning_info "      20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }
  if {$mom_out_angle_pos(0)<-30.0} {
    set mom_warning_info "      -20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }
  if {$mom_out_angle_pos(1)>270.0} {
    set mom_warning_info " C     20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }
  if {$mom_out_angle_pos(1)<-270.0} {
    set mom_warning_info " C     -20"
    set fl 1
    MOM_set_line_leader once " !Error! "
  }

  if {$mom_out_angle_pos(0)>0.0} {
    if {$mom_out_angle_pos(0)>$mom_kin_4th_axis_max} { set mom_kin_4th_axis_max $mom_out_angle_pos(0) ; }
  } elseif {$mom_out_angle_pos(0)<0.0} {
           if {$mom_out_angle_pos(0)<$mom_kin_4th_axis_min} { set mom_kin_4th_axis_min $mom_out_angle_pos(0) ; }
        }

  if {$mom_out_angle_pos(1)>0.0} {
    if {$mom_out_angle_pos(1)>$mom_kin_5th_axis_max} { set mom_kin_5th_axis_max $mom_out_angle_pos(1) ; }
  } elseif {$mom_out_angle_pos(1)<0.0} {
  	   if {$mom_out_angle_pos(1)<$mom_kin_5th_axis_min} { set mom_kin_5th_axis_min $mom_out_angle_pos(1) ; }
         }

  MOM_do_template rapid_xy
  MOM_do_template rapid_z

  if {$fl==1} {
     #MOM_output_to_listing_device $mom_warning_info
     MOM_catch_warning
     MOM_set_line_leader off " !Error! "
     set fl 0
  }

}


proc  cycle_set {} {
#_______________________________________________________________________________
# This procedure is executed for each new cycle.  This will make sure all
# non-modal addresses and Cycle parameters are output that need to be output
# at the start of a new cycle.
#_______________________________________________________________________________
  global mom_cycle_retract_mode
  global g_return

  MOM_force once G_motion X Z R cycle_dwell cycle_nodrag cycle_step

  if {$mom_cycle_retract_mode == "MANUAL"} {
      set g_return 99
  } elseif {$mom_cycle_retract_mode == "AUTO"} {
      set g_return 98
  } else {
      set g_return 99
  }
}


proc MOM_cycle_plane_change {} {
#_______________________________________________________________________________
# This procedure is executed at the end of cycle motion when there is a cycle
# move occuring at a higher level than the previous position.
#_______________________________________________________________________________
  MOM_do_template cycle_plane_change
}


proc MOM_cycle_off {} {
#_______________________________________________________________________________
# This procedure is executed at the end of cycle motion.
#_______________________________________________________________________________
  MOM_set_address_format G_motion Register
  MOM_do_template cycle_off
}

proc MOM_drill {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill cycle.
#_______________________________________________________________________________
  cycle_set
}

proc MOM_drill_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_drill
}

proc MOM_drill_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill dwell cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_drill_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill dwell cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_drill_dwell
}

proc MOM_drill_counter_sink {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill cycle.
#_______________________________________________________________________________
         MOM_drill
}

proc MOM_drill_counter_sink_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill cycle.
#_______________________________________________________________________________
  MOM_drill_move
}

proc MOM_drill_csink_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill dwell cycle.
#_______________________________________________________________________________
	 MOM_drill_dwell
}

proc MOM_drill_csink_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill dwell cycle.
#_______________________________________________________________________________
  MOM_drill_dwell_move
}

proc MOM_drill_deep {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a deep cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_drill_deep_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a  deep cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_drill_deep
}

proc MOM_drill_break_chip {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a brkchp cycle.
#_______________________________________________________________________________
       	 cycle_set
}

proc MOM_drill_break_chip_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a  brkchp cycle.
#_______________________________________________________________________________
  MOM_set_address_format G_motion Coordinate
  MOM_do_template cycle_breakchip
}

proc MOM_tap {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a tap cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_tap_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a tap cycle.
#_______________________________________________________________________________

  global mom_spindle_direction

  global cycle_tap_code
  global mom_sys_cycle_tap_code_left
  global mom_sys_cycle_tap_code

  if {$mom_spindle_direction == "CCLW"} {
      MOM_set_address_format G_motion Coordinate
      set cycle_tap_code $mom_sys_cycle_tap_code_left
  } else {
      set cycle_tap_code $mom_sys_cycle_tap_code
  }
  MOM_do_template cycle_tap
}

proc MOM_bore {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore
}

proc MOM_bore_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore dwell cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_dwell
}

proc MOM_bore_drag {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore drag cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_drag_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore_drag cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_drag
}

proc MOM_bore_no_drag {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore nodrag cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_no_drag_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore nodrag cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_no_drag
}

proc MOM_bore_back {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore back cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_back_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a back bore cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_back
}

proc MOM_bore_manual {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore manual cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_manual_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a manual bore cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_manual
}

proc MOM_bore_manual_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore manual dwell bore cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_manual_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a manual dwell borecycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_manual_dwell
}

proc MOM_load_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the LOAD/TOOL command.  All output for the LOAD
# command will take place with the initial move.
#_______________________________________________________________________________
  global  korr_move ; set  korr_move   1
#  MOM_force once  Z
#  MOM_do_template  tool_change
#  MOM_output_literal "(UAO,1)"
}


proc MOM_set_modes {} {
#_______________________________________________________________________________
# This procedure is executed for the Set Mode command.
#_______________________________________________________________________________
  global mom_output_mode
  global mom_arc_mode
  global mom_feed_rate_output_mode
  global mom_parallel_to_axis
  global mom_machine_mode
  global mom_modes_text
  global g_output_mode

# "ABSOLUTE", "INCREMENTAL"
  if {$mom_output_mode == "ABSOLUTE"} {
      MOM_incremental OFF X Y Z A B
  } elseif {$mom_output_mode == "INCREMENTAL"} {
      MOM_incremental ON X Y Z A B
  }

# "LINEAR", "CIRCULAR"

  if {$mom_arc_mode == "LINEAR"} {
  } elseif {$mom_arc_mode == "CIRCULAR"} {
  }

# "OFF", "IPM", "IPR", "MMPM", "MMPR", "INVERSE"

  if {$mom_feed_rate_output_mode == "FPM"} {
  } elseif {$mom_feed_rate_output_mode == "FPR"} {
  } elseif {$mom_feed_rate_output_mode == "INVERSE"} {
  } elseif {$mom_feed_rate_output_mode == "OFF"} {
  }

# "ZAXIS", "WAXIS", "UAXIS"

  if {$mom_parallel_to_axis == "ZAXIS"} {
  } elseif {$mom_parallel_to_axis == "WAXIS"} {
  } elseif {$mom_parallel_to_axis == "UAXIS"} {
  }

# "MILL", "TURN", "PUNCH", "LASER", "TORCH", "WIRE"

  if {$mom_machine_mode == "MILL"} {
  } elseif {$mom_machine_mode == "TURN"} {
  } elseif {$mom_machine_mode == "PUNCH"} {
  } elseif {$mom_machine_mode == "LASER"} {
  } elseif {$mom_machine_mode == "TORCH"} {
  } elseif {$mom_machine_mode == "WIRE"} {
  }
}

proc MOM_cutcom_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Cutcom command is activated.
#_______________________________________________________________________________
 global mom_cutcom_status   mom_cutcom_mode
 if {[info exists mom_cutcom_mode]} { set mom_cutcom_status $mom_cutcom_mode }
}

proc MOM_cutcom_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Cutcom command is deactivated.
#_______________________________________________________________________________
  global mom_cutcom_status mom_cutcom_mode
  catch {unset mom_cutcom_mode}
  MOM_do_template cutcom_off
}

proc MOM_spindle_rpm {} {
#_______________________________________________________________________________
# This procedure is executed when the Spindle command is activated.
#_______________________________________________________________________________
  global mom_spindle_speed
  global spindle_speed  initial_move  operation
  set  spindle_speed  1
 #  if {$spindle_speed == 0} {
 #      set   spindle_speed_1  $mom_spindle_speed
 #      set  spindle_speed  1
 #  } else {
 #       set   spindle_speed_2  $mom_spindle_speed
 # }
#
#  if {$initial_move == 1 && $mom_spindle_speed != 0} {
      MOM_do_template spindle_start
      MOM_do_template spindle_start_1
#  }
}


proc MOM_spindle_css {} {
#_______________________________________________________________________________
  MOM_spindle_rpm
}

proc MOM_spindle_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Spindle command is deactivated.
#_______________________________________________________________________________
  global  mom_spindle_speed spindle_speed
  MOM_do_template spindle_off
  set  spindle_speed  0
}

proc MOM_coolant_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is activated.
#_______________________________________________________________________________
 global mom_coolant_status mom_coolant_mode
 set mom_coolant_status $mom_coolant_mode
}

proc MOM_coolant_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is deactivated.
#_______________________________________________________________________________
  MOM_do_template coolant_off
}

proc MOM_opstop {} {
#_______________________________________________________________________________
# This procedure is executed when the Optional Stop command is activated.
#_______________________________________________________________________________
  MOM_do_template opstop
}

proc MOM_operator_message {} {
#_______________________________________________________________________________
# This procedure is executed when the Operator Message command is activated.
#_______________________________________________________________________________
  global mom_operator_message
  MOM_suppress once N
  if {$mom_operator_message != "ON" && $mom_operator_message != "OFF"} {
      MOM_output_literal "$mom_operator_message"
  }
}

proc MOM_stop {} {
#_______________________________________________________________________________
# This procedure is executed when the Program Stop command is activated.
#_______________________________________________________________________________
         MOM_do_template stop
}


proc MOM_delay {} {
#_______________________________________________________________________________
# This procedure is executed when the Delay command is activated.
#_______________________________________________________________________________
         MOM_do_template delay
         MOM_do_template delay_1
}

proc MOM_auxfun {} {
#_______________________________________________________________________________
# This procedure is executed when the Auxiliary command is activated.
#_______________________________________________________________________________
  MOM_do_template auxiliary
}

proc MOM_prefun {} {
#_______________________________________________________________________________
# This procedure is executed when the Preparatory function command is activated.
#_______________________________________________________________________________
  MOM_do_template preparatory
}

proc MOM_length_compensation {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Length Compensation command is activated.
#_______________________________________________________________________________
}

proc MOM_tool_preselect {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Preselect command is activated.
#_______________________________________________________________________________
}

proc MOM_text {} {
#_______________________________________________________________________________
# This procedure is executed when the Text command is activated.
#_______________________________________________________________________________
  global mom_user_defined_text mom_record_fields
  global mom_field_count
  global mom_sys_control_out mom_sys_control_in
  global mom_record_text mom_pprint mom_Instruction mom_operator_message
  global mom_pprint_defined mom_operator_message_defined
  global mom_warning_info

  global output_mode

  switch [string toupper $mom_record_fields(0)] {
  "SET" { ; }
  "PPRINT" {
           set mom_pprint_defined 1
           set mom_pprint $mom_record_text
           MOM_pprint
           }
  "DISPLY" {
           set mom_operator_message_defined 1
           set mom_operator_message $mom_record_text
           MOM_operator_message
          }
  "INSERT" {
           if {[info exists mom_record_fields(1)]}  {
              set mom_Instruction [string toupper $mom_record_text ] ; #$mom_record_fields(1)] ;
              MOM_insert
              set mom_warning_info "WARNING:    'INSERT/$mom_Instruction' -  "
              MOM_output_to_listing_device $mom_warning_info  ; #  MOM_catch_warning ;
           }
         }
  "ZERO" {
          global mom_work_coordinate_number
          set mom_work_coordinate_number $mom_record_fields(1)
          MOM_zero
         }
 default {
            set mom_warning_info "WARNING:   !($mom_user_defined_text)\n"
            MOM_output_to_listing_device $mom_warning_info  ; #  MOM_catch_warning ;
            MOM_output_literal $mom_user_defined_text
         }
  }

}


#=============================================================
proc  MOM_zero {} {
#=============================================================
#     G      .
#         Fanuc.
#   G   ZERO/CHECK, ZERO/ON, ZERO/OFF, ZERO/THRU,   G   ZERO/n.
#
  global mom_sys_wcs_change_code mom_work_coordinate_number
  if {[info exists mom_work_coordinate_number]==0} { set mom_work_coordinate_number 0 ; }
  if {$mom_work_coordinate_number==""} { set mom_work_coordinate_number 0 }
  MOM_output_literal "(UAO,$mom_work_coordinate_number)"

}

#------------------------------------------------------------------------------
# Local Procedures for use in this post
#------------------------------------------------------------------------------

proc CHECK_PLANE {} {

  global mom_tool_axis
  global mom_path_name

  global prev_error_path

#-------- Decision on plane of operation

  if {[EQ_is_equal $mom_tool_axis(1) 1.0]} {
      set g_plane 18
  } elseif {[EQ_is_equal $mom_tool_axis(2) 0.0]} {
      set g_plane 19
  } elseif {[EQ_is_equal $mom_tool_axis(2) 1.0]} {
      set g_plane 17
  } elseif {([EQ_is_gt $mom_tool_axis(2) 0.0] && \
                 [EQ_is_lt $mom_tool_axis(2) 1.0]) || \
             [EQ_is_lt $mom_tool_axis(2) 0.0] } {
      set g_plane "NONE"

      set i [format "%-.7f" $mom_tool_axis(0)]
      set j [format "%-.7f" $mom_tool_axis(1)]
      set k [format "%-.7f" $mom_tool_axis(2)]

      # Check to output this error only once per path

  } else {
      set g_plane 18
  }

  return $g_plane
}


################################################################################

#***************************
# Source in user's tcl file.
#***************************
 set user_tcl_file ${cam_post_dir}alpha_users.tcl
 if { [file exists $user_tcl_file] } {
    source $user_tcl_file
 }
